! avl.f90
! Public domain 2007-2011 James Van Buskirk
! Compiled with:
! gfortran -Wall -mwindows avl.f90 -oavl -lgdi32 -lopengl32
module mytype
   implicit none
   type QQ
      integer x
   end type QQ
   interface operator(<=)
      module procedure lessOrEqual
   end interface operator(<=)
   interface assignment(=)
      module procedure assign
   end interface assignment(=)
   contains
      function lessOrEqual(x,y)
         logical lessOrEqual
         type(QQ), intent(in) :: x
         type(QQ), intent(in) :: y
         lessOrEqual = x%x <= y%x
      end function lessOrEqual
      subroutine assign(x,y)
         type(QQ),intent(out) :: x
         type(QQ),intent(in) :: y
         x%x = y%x
      end subroutine assign
end module mytype

module avl
   use mytype
   implicit none
   type QQNode
      type(QQnode),pointer :: left=>NULL()
      type(QQnode),pointer :: right=>NULL()
      type(QQ) data
      integer :: balance = 0
   end type QQNode
   contains
   subroutine avlRotateLeft(root,delta)
      type(QQnode),pointer :: root
      integer delta
      type(QQnode),pointer :: temp
      temp=>root
      root=>root%right
      temp%right=>root%left
      root%left=>temp
      select case(root%left%balance)
         case(1)
            select case(root%balance)
               case(-1)
                  root%left%balance = 0
                  root%balance = -2
                  delta = 0
               case(0)
                  root%left%balance = 0
                  root%balance = -1
                  delta = 0
               case(1)
                  root%left%balance = -1
                  root%balance = -1
                  delta = 0
               case default
                  write(*,'(a)') 'Bad case in avlRotateLeft'
                  stop
            end select
         case(2)
            select case(root%balance)
               case(0)
                  root%left%balance = 1
                  root%balance = -1
                  delta = 0
               case(1)
                  root%left%balance = 0
                  root%balance = 0
                  delta = -1
               case(2)
                  root%left%balance = -1
                  root%balance = 0
                  delta = -1
               case default
                  write(*,'(a)') 'Bad case in avlRotateLeft'
                  stop
            end select
         case default
            write(*,'(a)') 'Bad case in avlRotateLeft'
            stop
      end select
   end subroutine avlRotateLeft

   subroutine avlRotateRight(root,delta)
      type(QQnode),pointer :: root
      integer delta
      type(QQnode),pointer :: temp
      temp=>root
      root=>root%left
      temp%left=>root%right
      root%right=>temp
      select case(root%right%balance)
         case(-1)
            select case(root%balance)
               case(1)
                  root%right%balance = 0
                  root%balance = 2
                  delta = 0
               case(0)
                  root%right%balance = 0
                  root%balance = 1
                  delta = 0
               case(-1)
                  root%right%balance = 1
                  root%balance = 1
                  delta = 0
               case default
                  write(*,'(a)') 'Bad case in avlRotateRight'
                  stop
            end select
         case(-2)
            select case(root%balance)
               case(0)
                  root%right%balance = -1
                  root%balance = 1
                  delta = 0
               case(-1)
                  root%right%balance = 0
                  root%balance = 0
                  delta = -1
               case(-2)
                  root%right%balance = 1
                  root%balance = 0
                  delta = -1
               case default
                  write(*,'(a)') 'Bad case in avlRotateRight'
                  stop
            end select
         case default
            write(*,'(a)') 'Bad case in avlRotateRight'
            stop
      end select
   end subroutine avlRotateRight

   recursive subroutine avlInsert(root,x,delta)
      type(QQnode),pointer :: root
      type(QQ) x
      integer delta
      if(.NOT.associated(root)) then
         allocate(root)
         root%data = x
         delta = 1
         return
      end if
      if(x <= root%data) then
         if(root%data <= x) then
            write(*,'(a)') 'Duplicate entry in avlInsert'
            delta = 0
            return
         end if
         call avlInsert(root%left,x,delta)
         if(delta==1) then
            root%balance = root%balance-1
            if(root%balance > -2) then
               delta = -root%balance
               return
            end if
            if(root%left%balance == 1) then
               call avlRotateLeft(root%left,delta)
            end if
            call avlRotateRight(root,delta)
            delta = 0
         end if
         return
      else ! x > root%data
         call avlInsert(root%right,x,delta)
         if(delta==1) then
            root%balance = root%balance+1
            if(root%balance < 2) then
               delta = root%balance
               return
            end if
            if(root%right%balance == -1) then
               call avlRotateRight(root%right,delta)
            end if
            call avlRotateLeft(root,delta)
            delta = 0
         end if
         return
      end if
   end subroutine avlInsert

   recursive subroutine avlDelete(root,x,delta)
      type(QQnode),pointer :: root
      type(QQ) x
      integer delta
      type(QQnode),pointer :: temp
      if(.NOT.associated(root)) then
         write(*,'(a)') 'Entry not found in avlDelete'
         delta = 0
         return
      end if
      if(x <= root%data) then
         if(root%data <= x) then
            if(associated(root%left)) then
               call avlSwapLargestLeft(root%left,root%data,delta)
            else ! root or root%right is a leaf node
               temp=>root
               root=>root%right
               deallocate(temp)
               delta = -1
               return
            end if
         else ! x < root%data
            call avlDelete(root%left,x,delta)
         end if
         ! Now handle cleanup after avlSwapLargestLeft
         ! or recursive invocation of avlDelete
         if(delta == -1) then
            root%balance = root%balance+1
            if(root%balance < 2) then
               delta = root%balance-1
               return
            end if
            if(root%right%balance == -1) then
               call avlRotateRight(root%right,delta)
            end if
            call avlRotateLeft(root,delta)
            return
         end if
      else ! x > root%data
         call avlDelete(root%right,x,delta)
         ! Now handle cleanup after recursive invocation
         ! of avlDelete
         if(delta == -1) then
            root%balance = root%balance-1
            if(root%balance > -2) then
               delta = -root%balance-1
               return
            end if
            if(root%left%balance == 1) then
               call avlRotateLeft(root%left,delta)
            end if
            call avlRotateRight(root,delta)
            return
         end if
      end if
   end subroutine avlDelete

   recursive subroutine avlSwapLargestLeft(root,x,delta)
      type(QQnode),pointer :: root
      type(QQ) x
      integer delta
      type(QQnode),pointer :: temp
      if(associated(root%right)) then
         call avlSwapLargestLeft(root%right,x,delta)
      else ! root or root%left is a leaf node
         x = root%data
         temp => root
         root => root%left
         deallocate(temp)
         delta = -1
         return
      end if
      ! Now handle cleanup after recursive invocation
      ! of avlSwapLargestLeft
      if(delta == -1) then
         root%balance = root%balance-1
         if(root%balance > -2) then
            delta = -root%balance-1
            return
         end if
         if(root%left%balance == 1) then
            call avlRotateLeft(root%left,delta)
         end if
         call avlRotateRight(root,delta)
         return
      end if
   end subroutine avlSwapLargestLeft

   recursive subroutine avlClear(root)
      type(QQnode),pointer :: root
      if(.NOT.associated(root)) then
         return
      end if
      if(associated(root%left)) then
         call avlClear(root%left)
      end if
      if(associated(root%right)) then
         call avlClear(root%right)
      end if
      deallocate(root)
   end subroutine avlClear
end module avl

module allfun
   use win32_types
   use kernel32
   use user32
   use gdi32
   use comdlg32
   use wgl
   use opengl_types
   use gl
   use ISO_C_BINDING
   use mytype
   use avl
   implicit none
   integer,save :: counter = 0
   integer(GLuint),save :: SystemFont = 0
   type(GLYPHMETRICSFLOAT_T),save :: gmf(256)
   real(glFloat),parameter :: sc = 0.044
   real(glFloat),parameter :: pi = 4*atan(1.0)
   type(QQnode),pointer,save :: root=>NULL()
   integer,parameter :: RED = -1
   integer,parameter :: GREEN = 0
   integer,parameter :: BLUE = 1
   integer,parameter :: BLACK = 2
   real(GLfloat),parameter :: radius = 0.02
   integer,save :: iunit
   logical, save :: skip = .FALSE.
   contains
      function WndProc(hwnd_, iMsg, wParam_, lParam_) bind(C)
!GCC$ ATTRIBUTES STDCALL :: WndProc
         integer(LRESULT) WndProc
         integer(HWND), value :: hwnd_
         integer(UINT), value :: iMsg
         integer(WPARAM), value :: wParam_
         integer(LPARAM), value :: lParam_
         integer(HDC) hdc_
         type(RECT_T) rect
         type(RECT_T),pointer :: NULL_RECT
         integer(C_INT) result4
         integer width,height
         integer(C_INTPTR_T) result8
!         integer(HGLRC) hrc

         select case(iMsg)
            case(WM_CREATE)
               call OpenInput(hwnd_)
               hdc_ = GetDC(hwnd_)
               call myPixelFormatSetupFunction(hdc_,iany([PFD_DRAW_TO_WINDOW, &
                  PFD_SUPPORT_OPENGL,PFD_DOUBLEBUFFER]),32_BYTE,4_BYTE)
               call StartUp(hdc_)
               WndProc = 0
               return

            case(WM_PAINT)
               hdc_ = wglGetCurrentDC()
               result4 = GetClientRect(hwnd_, rect)
               if(all([rect%right,rect%bottom] > 0)) then
                  call glClearColor(1.0,1.0,1.0,1.0)
                  call glClear(GL_COLOR_BUFFER_BIT)
                  call draw_tree(hdc_,rect,0,root)
                  result4 = SwapBuffers(hdc_)
               end if
               result4 = ValidateRgn(hwnd_,0_HRGN)
               WndProc = 0
               return

            case(WM_CLOSE)
               hdc_ = wglGetCurrentDC()
               call CleanUp(SystemFont)
               result4 = ReleaseDC(hwnd_,hdc_)
               WndProc = DefWindowProc(hwnd_,iMsg,wParam_,lParam_)
               return

            case(WM_DESTROY)
               call PostQuitMessage(0)
               WndProc = 0
               return

            case(WM_ERASEBKGND)
               WndProc = BOOL_TRUE
               return

            case(WM_SIZE)
               width = int(ibits(lParam_,0,bit_size(0_WORD)),GLsizei)
               height = int(ibits(lParam_,bit_size(0_WORD),bit_size(0_WORD)),GLsizei)
               if(all([width,height] > 0)) then
                  call resize(width,height)
                  result4 = InvalidateRgn(hwnd_,0_HRGN,BOOL_FALSE)
                  WndProc = 0
                  return
               end if

            case(WM_KEYDOWN)
               select case(wParam_)
                  case(13) ! ENTER
                     call advance_counter(hwnd_)
                     call C_F_POINTER(C_NULL_PTR,NULL_RECT)
                     result4 = InvalidateRect(hwnd_,NULL_RECT,1)
                  case(27) ! ESC
                     result8 = SendMessage(hwnd_,WM_CLOSE,int(0,WPARAM),int(0,LPARAM))
!                     call PostQuitMessage(0)
                     WndProc = 0
                     return
                  case(87) ! W key
                     hdc_ = wglGetCurrentDC()
                     call CleanUp(SystemFont)
                     result4 = ReleaseDC(hwnd_,hdc_)
                     call drawBmp(hdc_,rect)
                     hdc_ = GetDC(hwnd_)
                     call myPixelFormatSetupFunction(hdc_,iany([PFD_DRAW_TO_WINDOW, &
                        PFD_SUPPORT_OPENGL,PFD_DOUBLEBUFFER]),32_BYTE,4_BYTE)
                     call StartUp(hdc_)
                     result4 = GetClientRect(hwnd_, rect)
                     width = rect%right-rect%left
                     height = rect%bottom-rect%top
                     if(all([width,height] > 0)) then
                        call resize(width,height)
                        WndProc = 0
                        return
                     end if
                  case(88) ! X key
                     skip = .TRUE.
                     call advance_counter(hwnd_)
                     call C_F_POINTER(C_NULL_PTR,NULL_RECT)
                     result4 = InvalidateRect(hwnd_,NULL_RECT,1)
               end select
         end select

         WndProc = DefWindowProc(hwnd_, iMsg, wParam_, lParam_)
      end function WndProc

      subroutine OpenInput(hwnd_)
         integer(HWND),value :: hwnd_
         type(OPENFILENAME_T) lpofn
         integer result4
         character(kind=C_CHAR),target :: lpstrFilter*(80)
         character(kind=C_CHAR),target :: lpstrCustomFilter*(80)
         character(kind=C_CHAR),target :: lpstrFile*(256)
         character(kind=C_CHAR),target :: lpstrFileTitle*(256)
         character(kind=C_CHAR),target :: lpstrInitialDir*(256)
         character(kind=C_CHAR),target :: lpstrDefExt*(3)

         lpstrFilter = 'Text Files'//achar(0)//'*.txt;*.dat'// &
            achar(0)//achar(0)
         lpstrFile = achar(0)
         result4 = GetCurrentDirectory(len(lpstrInitialDir), &
            lpstrInitialDir)
         lpstrDefExt = 'dat'
         lpofn = OPENFILENAME_T( &
            lStructSize = int(C_SIZEOF(lpofn),DWORD), &
            hwndOwner = hwnd_, &
            hInstance = 0_HINSTANCE, &
            lpstrFilter = C_LOC(lpstrFilter(1:1)), &
            lpstrCustomFilter = C_LOC(lpstrCustomFilter(1:1)), &
            nMaxCustFilter = len(lpstrCustomFilter), &
            nFilterIndex = 1, &
            lpstrFile = C_LOC(lpstrFile(1:1)), &
            nMaxFile = len(lpstrFile), &
            lpstrFileTitle = C_LOC(lpstrFileTitle(1:1)), &
            nMaxFileTitle = len(lpstrFileTitle), &
            lpstrInitialDir = C_LOC(lpstrInitialDir(1:1)), &
            lpstrTitle = C_NULL_PTR, &
            Flags = iany([OFN_CREATEPROMPT,OFN_DONTADDTORECENT, &
               OFN_FILEMUSTEXIST,OFN_PATHMUSTEXIST]), &
            nFileOffset = 0, &
            nFileExtension = 0, &
            lpstrDefExt = C_LOC(lpstrDefExt(1:1)), &
            lCustData = 0_LPARAM, &
            lpfnHook = C_NULL_FUNPTR, &
            lpTemplateName = C_NULL_PTR, &
            pvReserved = C_NULL_PTR, &
            dwReserved = 0, &
            FlagsEx = 0)
         result4 = GetOpenFileName(lpofn)
         open(newunit=iunit, &
            file=lpstrFileTitle(1:index(lpstrFileTitle,achar(0))-1), &
            status='old')
      end subroutine OpenInput

      subroutine advance_counter(hwnd_)
         integer(HWND),value :: hwnd_
         type(QQ) x
         integer delta
         character command
         integer(LRESULT) result8
         character(80) line
         do
            read(iunit,'(a)',advance='no',end=10) command
            select case(command)
               case('i','I')
                  read(iunit,'(i10)') x%x
                  write(*,'(a,i0)') 'Inserting: ',x%x
                  call avlInsert(root,x,delta)
                  counter = counter+1
               case('d','D')
                  read(iunit,'(i10)') x%x
                  write(*,'(a,i0)') 'Deleting: ',x%x
                  call avlDelete(root,x,delta)
                  counter = counter-1
               case('c','C')
                  read(iunit,'()')
                  write(*,'(a,i0)') 'Clearing'
                  call avlClear(root)
                  counter = 0
               case('!')
                  read(iunit,'(a)') line
                  write(*,'(a)') '!'//trim(line)
                  skip = .FALSE.
                  cycle
            end select
            if(.NOT.skip) exit
         end do
         return
10       continue
         result8 = SendMessage(hwnd_,WM_CLOSE,int(0,WPARAM),int(0,LPARAM))
      end subroutine advance_counter

      subroutine StartUp(hdc_)
         integer(HDC),value :: hdc_
         integer(C_INT) result4
         integer width,height
         integer(HGLRC) hrc

         hrc = wglCreateContext(hdc_)
         result4 = wglMakeCurrent(hdc_,hrc)
         width = GetDeviceCaps(hdc_,HORZRES)
         height = GetDeviceCaps(hdc_,VERTRES)
         call MakeFont(hdc_,SystemFont,min(width,height)/70, &
            'Lucida Sans Unicode'//achar(0),FF_SWISS,BOOL_FALSE, &
            ANSI_CHARSET,500)
         call glCullFace(GL_BACK)
         call glEnable(GL_CULL_FACE)
         call glEnable(GL_LINE_SMOOTH)
         call glHint(GL_LINE_SMOOTH_HINT,GL_NICEST)
         call glEnable(GL_POLYGON_SMOOTH)
         call glHint(GL_POLYGON_SMOOTH_HINT,GL_NICEST)
         call glEnable(GL_BLEND)
         call glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
      end subroutine StartUp

      subroutine resize(width,height)
         integer(GLsizei),value :: width,height
         integer(GLint) viewportX
         integer(GLint) viewportY
         real(GLdouble) left
         real(GLdouble) right
         real(GLdouble) top
         real(GLdouble) bottom
         real(GLdouble) zNear
         real(GLdouble) zFar
         if(width >= height) then
            zNear = -1
            top = 1
            left = -(width-height)/(2.0_glDouble*height)
            right = 1-left
            bottom = 0
            zFar = 1
            call glMatrixMode(GL_PROJECTION)
            call glLoadIdentity()
            call glOrtho(left,right,bottom,top,zNear,zFar)
            call glMatrixMode(GL_MODELVIEW)
            viewportX = 0
            viewportY = 0
            call glViewport(viewportX,viewportY,width,height)
         else
            zNear = -1
            right = 1
            left = 0
            bottom = -(height-width)/(2.0_glDouble*width)
            top = 1-bottom
            zFar = 1
            call glMatrixMode(GL_PROJECTION)
            call glLoadIdentity()
            call glOrtho(left,right,bottom,top,zNear,zFar)
            call glMatrixMode(GL_MODELVIEW)
            viewportX = 0
            viewportY = 0
            call glViewport(viewportX,viewportY,width,height)
         end if
      end subroutine resize

      subroutine CleanUp(Font1)
         integer(GLuint) Font1
         integer(HGLRC) hrc
         integer(HDC) hdc_
         integer(C_INT) result4

         hrc = wglGetCurrentContext()
         if(hrc /= 0) then
            call glDeleteLists(Font1,256)
            Font1 = 0
            ! Obtain its associated deviced context
            hdc_ = wglGetCurrentDC()
            ! Make the rendering context not current
            result4 = wglMakeCurrent(0_HDC,0_HGLRC)
            ! Delete the rendering context
            result4 = wglDeleteContext(hrc)
         end if
      end subroutine CleanUp

      recursive subroutine MakeFont(hdc_,FontBase,height,name, &
         face,italic,charset,weight) bind(C)
         integer(HDC),value :: hdc_
         integer(GLuint) FontBase
         integer(C_INT),value :: height
         character(kind=C_CHAR) name(*)
         integer(DWORD),value :: face
         integer(BOOL),value :: italic
         integer(DWORD),value :: charset
         integer(C_INT),value :: weight
         integer(HFONT) fonts
         integer(C_INT) result4
         integer(HGDIOBJ) oldfont
         integer(HGDIOBJ) result_h

         if(FontBase == 0) FontBase = glGenLists(256_GLsizei)
         fonts = CreateFont( &
            nHeight = height, &
            nWidth = 0_C_INT, &
            nEscapement = 0_C_INT, &
            nOrientation = 0_C_INT, &
            fnWeight = weight, &
            fdwItalic = italic, &
            fdwUnderline = BOOL_FALSE, &
            fdwStrikeOut = BOOL_FALSE, &
            fdwCharSet = charset, &
            fdwOutputPrecision = OUT_TT_PRECIS, &
            fdwClipPrecision = CLIP_MASK, &
            fdwQuality = ANTIALIASED_QUALITY, &
            fdwPitchAndFamily = face, &
            lpszFace = name)
         oldfont = SelectObject(hdc_,fonts)
         result4 = wglUseFontOutlines( &
            hdc_ = hdc_, &
            first = 0_DWORD, &
            count = 256_DWORD, &
            listBase = FontBase, &
            deviation = 0.0, &
            extrusion = 0.0, &
            format = WGL_FONT_POLYGONS, &
            lpgmf = gmf)
         result_h = SelectObject(hdc_,oldfont)
      end subroutine MakeFont

      recursive subroutine myPixelFormatSetupFunction(hdc_, &
         dwFlags,cColorBits,cAuxBuffers) bind(C)
         integer(HDC),value :: hdc_
         integer(DWORD),value :: dwFlags
         type(PIXELFORMATDESCRIPTOR_T) pfd
         integer(C_INT) format
         integer(C_INT) result4
         integer(BYTE),value :: cColorBits
         integer(BYTE),value :: cAuxBuffers

         pfd = PIXELFORMATDESCRIPTOR_T( &
            nSize = int(C_SIZEOF(pfd),WORD), &
            nVersion = 1_WORD, &
            dwFlags = dwFlags, &
            iPixelType = PFD_TYPE_RGBA, &
!            cColorBits = 32_BYTE, &
            cColorBits = cColorBits, &
            cRedBits = 0_BYTE, &
            cRedShift = 0_BYTE, &
            cGreenBits = 0_BYTE, &
            cGreenShift = 0_BYTE, &
            cBlueBits = 0_BYTE, &
            cBlueShift = 0_BYTE, &
! Did not display properly when cAlphaBits = 8
! and video mode switched from 32 bpp to 16 bpp
            cAlphaBits = 8_BYTE, &
!            cAlphaBits = 0_BYTE, &
            cAlphaShift = 0_BYTE, &
            cAccumBits = 64_BYTE, &
            cAccumRedBits = 0_BYTE, &
            cAccumGreenBits = 0_BYTE, &
            cAccumBlueBits = 0_BYTE, &
            cAccumAlphaBits = 0_BYTE, &
            cDepthBits = 24_BYTE, &
            cStencilBits = 8_BYTE, &
            cAuxBuffers = cAuxBuffers, &
            iLayerType = PFD_MAIN_PLANE, & ! Ignored.
            bReserved = 0_BYTE, &
            dwLayerMask = 0_DWORD, &
            dwVisibleMask = 0_DWORD, &
            dwDamageMask = 0_DWORD)
         format = ChoosePixelFormat(hdc_,pfd)
         result4 = SetPixelFormat(hdc_,format,pfd)
         result4 = DescribePixelFormat(hdc_,format, &
            int(C_SIZEOF(pfd),UINT),pfd)
!         if(iand(pfd%dwFlags,PFD_NEED_PALETTE) /= 0) then
!            call PaletteSetup(pfd)
!            if (hPalette_.NE.0_HPALETTE) then
!               result8 = SelectPalette(hdc_,hPalette_,BOOL_FALSE)
!               result4 = RealizePalette(hdc_)
!            end if
!         end if
      end subroutine myPixelFormatSetupFunction

      recursive subroutine draw_tree(hdc_,rect,position,root)
         integer(HDC),value :: hdc_
         type(RECT_T) rect
         integer,value :: position
         type(QQnode),pointer :: root
         real(GLfloat) x1,y1,x2,y2

         call getXY(position,x2,y2)
         if(position==0) then
            call circle(x2,y2,BLACK,0)
            if(associated(root)) then
               call draw_tree(hdc_,rect,1,root)
            end if
         else
            call getXY(position/2,x1,y1)
            call join(x1,y1,x2,y2)
            call circle(x2,y2,root%balance,root%data%x)
            if(associated(root%left)) then
               call draw_tree(hdc_,rect,2*position,root%left)
            end if
            if(associated(root%right)) then
               call draw_tree(hdc_,rect,2*position+1,root%right)
            end if
         end if
      end subroutine draw_tree

      subroutine getXY(position,x,y)
         integer,value :: position
         real x,y
         integer depth
         integer order
         if(position==0) then
            x = 0.5
            y = 1-1.1*radius
         else
            depth = bit_size(position)-leadz(position)
            order = iand(position,maskr(depth-1,kind(position)))
            y = 1-(depth+0.5)/7
            x = (order+0.5)/2**(depth-1)
         end if
      end subroutine getXY

      subroutine circle(x,y,color,number)
         real(GLfloat),value :: x, y
         integer,value :: color,number
         integer i
         real(GLfloat) theta
         character(kind=C_CHAR),target :: mess*80

         call glTranslatef(x,y,0.0)
         call glBegin(GL_LINE_LOOP)
         select case(color)
            case(RED)
               call glColor3f(0.8,0.0,0.0)
            case(GREEN)
               call glColor3f(0.0,0.8,0.0)
            case(BLUE)
               call glColor3f(0.0,0.0,0.8)
            case(BLACK)
               call glColor3f(0.0,0.0,0.0)
         end select
         do i = 0,359
            theta = pi*i/180
            call glVertex2f(radius*cos(theta),radius*sin(theta))
         end do
         call glEnd
         call glColor3f(0.0,0.0,0.0)
         call glTranslatef(-2*radius/3,-radius/2,0.0)
         if(color == BLACK) then
            mess = 'RT'
         else
            write(mess,'(i0)') number
         end if
         call glListBase(SystemFont)
!         call glTranslatef(0.5-0.5*mess_width*sc*0.8+0.5/400,0.01,0.0)
         call glScalef(sc*0.8,sc*0.8,sc*0.8)
         call glCallLists(len_trim(mess),GL_UNSIGNED_BYTE,C_LOC(mess(1:1)))
         call glLoadIdentity()
      end subroutine circle

      subroutine join(x1,y1,x2,y2)
         real(glFloat),value :: x1,y1,x2,y2
         real(Glfloat) x_dist,y_dist
         call glColor3f(0.0,0.0,0.0)
         x_dist = radius*(x2-x1)/hypot(x2-x1,y2-y1)
         y_dist = radius*(y2-y1)/hypot(x2-x1,y2-y1)
         call glBegin(GL_LINES)
         call glVertex2f(x1+x_dist,y1+y_dist)
         call glVertex2f(x2-x_dist,y2-y_dist)
         call glEnd
      end subroutine join

      subroutine drawBmp(hdc_,rect)
         integer(HDC),value :: hdc_
         type(RECT_T) rect
         integer(C_INT8_T),allocatable,target :: MEM(:)
         type(BITMAPINFO_T),pointer :: bmi
         type(RGBQUAD_T),pointer :: bmiColors(:)
         type(C_PTR) cptr
         integer(HBITMAP) hdcMem
         integer filesize
         integer offset
         integer(C_INT8_T),pointer :: image(:)
         integer(WORD) bpp
         integer nrows,ncols,row_len
         integer(HDC) BufferDC
         integer result4
         integer(HBITMAP) OldBitmap
         character(80) filename
         integer iunit
!         integer(WORD) bpp1
!         integer row_len1
!         integer(C_INT8_T),pointer :: image1(:)
!         integer i,j,k,k1

         if(hdc_*0==0) continue
         allocate(bmi)
         allocate(bmiColors(1))
         allocate(MEM(C_SIZEOF(bmi%bmiHeader)+2*C_SIZEOF(bmiColors(1))))
         deallocate(bmi)
         deallocate(bmiColors)
         cptr = C_LOC(MEM(1))
         call C_F_POINTER(cptr,bmi)
         cptr = C_LOC(bmi%bmiColors(1))
         call C_F_POINTER(cptr,bmiColors,[2])
         nrows = 400
         ncols = 400
         bpp = 24
         row_len = (ncols*bpp+31)/32*4
         bmi = BITMAPINFO_T( &
            bmiHeader = BITMAPINFOHEADER_T( &
               biSize = int(C_SIZEOF(bmi%bmiHeader),DWORD), &
               biWidth = ncols, &
               biHeight = nrows, &
               biPlanes = 1, &
               biBitCount = bpp, &
               biCompression = BI_RGB, &
               biSizeImage = row_len*nrows, &
               biXPelsPerMeter = 0, &
               biYPelsPerMeter = 0, &
               biClrUsed = 0, &
               biClrImportant = 0 &
            ), &
            bmiColors = bmiColors(1) &
         )
         bmiColors = [ &
            RGBQUAD_T(0,0,0,0), &
            RGBQUAD_T(-1,-1,-1,0) &
         ]
         BufferDC = CreateCompatibleDC(0_C_INTPTR_T)
         hdcMem = CreateDIBSection(BufferDC,bmi,DIB_RGB_COLORS,cptr,0_C_INTPTR_T,0)
         OldBitmap = SelectObject(BufferDC,hdcMem)
         rect = RECT_T(0,0,ncols,nrows)
         call myPixelFormatSetupFunction(BufferDC,iany([PFD_DRAW_TO_BITMAP, &
            PFD_SUPPORT_OPENGL]),24_BYTE,0_BYTE)
!            PFD_SUPPORT_OPENGL,PFD_SUPPORT_GDI]),32_BYTE,0_BYTE)
         call StartUp(BufferDC)
         call resize(ncols,nrows)
         call glClearColor(1.0,1.0,1.0,1.0)
         call glClear(GL_COLOR_BUFFER_BIT)
         call draw_tree(BufferDC,rect,0,root)
         call CleanUp(SystemFont)
         write(filename,'(a,i0,a)') 'avl',counter,'.bmp'
!         bpp1 = 8
!         row_len1 = (ncols*bpp1+31)/32*4
!         allocate(bmiColors(2**bpp1))
!         bmiColors = [(RGBQUAD_T(int(i,BYTE),int(i,BYTE),int(i,BYTE),0_BYTE), &
!            i=0,size(bmiColors)-1)]
!         allocate(image1(row_len1*nrows))
         call C_F_POINTER(cptr,image,[row_len*nrows])
!         do i = 0,nrows-1
!            do j = 0, ncols-1
!               k = i*row_len+bpp/8*j+1
!               k1 = i*row_len1+bpp1/8*j+1
!               image1(k1) = int((zext(image(k))+ &
!                  zext(image(k+1))+zext(image(k+2))+1)/3,BYTE)
!            end do
!         end do
         open(newunit=iunit,file=filename,status='replace',access='stream')
         write(iunit) int(Z'4D42',C_INT16_T)
         offset = 14+bmi%bmiHeader%biSize+ &
            int(C_SIZEOF(bmiColors(1)))*size(bmiColors)
!         filesize = offset+size(image1)
         filesize = offset+size(image)
         write(iunit) filesize
         write(iunit) 0_C_INT16_T
         write(iunit) 0_C_INT16_T
         write(iunit) offset
!         bmi%bmiHeader%biBitCount = bpp1
!         bmi%bmiHeader%biSizeImage = size(bmiColors)
!         bmi%bmiHeader%biClrUsed = size(bmiColors)
!         bmi%bmiHeader%biClrImportant = size(bmiColors)
!         bmi%bmiColors = bmiColors(1)
         write(iunit) bmi
         write(iunit) bmiColors(2:)
!         write(iunit) image1
         write(iunit) image
         close(iunit)
!         deallocate(bmiColors)
!         deallocate(image1)
         deallocate(MEM)
         result4 = DeleteDC(BufferDC)
         result4 = DeleteDC(hdcMem)
      end subroutine drawBmp

      function zext(x)
         integer(BYTE),value :: x
         integer zext
         zext = x
         zext = iand(zext,255)
      end function zext

      function WinMain(hInstance_, hPrevInstance, lpCmdLine, nCmdShow) bind(C, name='WinMain')
!GCC$ ATTRIBUTES STDCALL :: WinMain
         integer(C_INT) WinMain
         integer(HINSTANCE), value :: hInstance_
         integer(HINSTANCE), value :: hPrevInstance
         type(C_PTR), value :: lpCmdLine
         integer(C_INT), value :: nCmdShow
         character(kind=C_CHAR), pointer :: pcNull(:)
         type(WNDCLASSEX_T) WndClass
         character(kind=C_CHAR), pointer :: cDefault(:)
         character(kind=C_CHAR), target :: szAppName*(80)
         integer(C_SHORT) result2
         integer(HWND) hwnd_
         character(kind=C_CHAR), target :: szWindowCaption*(80)
         integer(C_INT) result4
         integer(C_INTPTR_T) result8
         type(MSG_T) msg
         type(RECT_T) rect
         integer(DWORD) ws

         if(hPrevInstance*0+nCmdShow*0 == 0) continue
         nullify(pcNull)
         call C_F_POINTER(lpCmdLine,cDefault,[0])
         szAppName = 'figs'//achar(0)
         WndClass%cbSize = size(transfer(Wndclass,[0_C_INT8_T]))
         WndClass%style = ior(CS_HREDRAW, CS_VREDRAW)
         WndClass%lpfnWndProc = C_FUNLOC(WndProc)
         WndClass%cbClsExtra = 0
         WndClass%cbWndExtra = 0
         WndClass%hInstance = hInstance_
         WndClass%hIcon = LoadIcon(0_C_INTPTR_T, cDefault) ! IDI_APPLICATION
         WndClass%hCursor = LoadCursor(0_C_INTPTR_T, cDefault) ! IDC_ARROW
         WndClass%hbrBackground = GetStockObject(WHITE_BRUSH)
         WndClass%lpszMenuName = C_NULL_PTR
         WndClass%lpszClassName = C_LOC(szAppName(1:1))
         WndClass%hIconSm = LoadIcon(0_C_INTPTR_T, cDefault) ! IDI_APPLICATION

         result2 = RegisterClassEx(WndClass)

         szWindowCaption = 'AVL Tree Test Program'//achar(0)
         rect = RECT_T(0,0,400,400)
         ws = WS_OVERLAPPEDWINDOW
         result4 = AdjustWindowRect(rect,ws,0)
         hwnd_ = CreateWindowEx(0, szAppName, szWindowCaption, &
            ws, CW_USEDEFAULT, CW_USEDEFAULT, rect%right-rect%left, &
            rect%bottom-rect%top, &
            0_C_INTPTR_T, 0_C_INTPTR_T, hInstance_, C_NULL_PTR)

         result4 = ShowWindow(hwnd_, SW_SHOWDEFAULT)
         result4 = UpdateWindow(hwnd_)

         do while(GetMessage(msg, 0_C_INTPTR_T, 0, 0) /= 0)
            result4 = TranslateMessage(msg)
            result8 = DispatchMessage(msg)
         end do

         call ExitProcess(int(msg%wParam, C_INT))
         WinMain = 0
      end function WinMain
end module allfun
